<?php
/************************************************************************
 * MXChange v0.2.1                                    Start: 12/16/2008 *
 * ===============                              Last change: 12/16/2008 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : filters.php                                      *
 * -------------------------------------------------------------------- *
 * Short description : Functions for filter system                      *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Funktionen fuer Filter-System                    *
 * -------------------------------------------------------------------- *
 * $Revision::                                                        $ *
 * $Date::                                                            $ *
 * $Tag:: 0.2.1-FINAL                                                 $ *
 * $Author::                                                          $ *
 * Needs to be in all Files and every File needs "svn propset           *
 * svn:keywords Date Revision" (autoprobset!) at least!!!!!!            *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2008 by Roland Haeder                           *
 * For more information visit: http://www.mxchange.org                  *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	$INC = substr(dirname(__FILE__), 0, strpos(dirname(__FILE__), '/inc') + 4) . '/security.php';
	require($INC);
}

// Init "generic filter system"
function initFilterSystem () {
	// Is the filter already initialized?
	if ((isset($GLOBALS['filters']['chains'])) && (is_array($GLOBALS['filters']['chains']))) {
		// Then abort here
		addFatalMessage(__FUNCTION__, __LINE__, getMessage('FILTER_FAILED_ALREADY_INIT'));
		return false;
	} // END - if

	// Init the filter system (just some ideas)
	$GLOBALS['filters']['chains'] = array(
		'preinit'   => array(), // Filters for pre-init phase
		'postinit'  => array(), // Filters for post-init phase
		'shutdown'  => array()  // Filters for shutdown phase
	);

	// Init loaded filters and counter
	$GLOBALS['filters']['loaded'] =  array();
	$GLOBALS['filters']['counter'] = array();

	// Load all saved filers if sql_patches is updated
	if (GET_EXT_VERSION('sql_patches') >= '0.5.9') {
		// Init add
		$add = '';
		if (GET_EXT_VERSION('sql_patches') >= '0.6.0') $add = ", `filter_counter`";

		// Load all active filers
		$result = SQL_QUERY("SELECT `filter_name`,`filter_function`,`filter_active`".$add."
FROM `{!_MYSQL_PREFIX!}_filters`
ORDER BY `filter_id` ASC", __FUNCTION__, __LINE__);

		// Are there entries?
		if (SQL_NUMROWS($result) > 0) {
			// Load all filters
			while ($filterArray = SQL_FETCHARRAY($result)) {
				// Get filter name and function
				$filterName     = $filterArray['filter_name'];
				$filterFunction = $filterArray['filter_function'];

				// Set counter to default
				$GLOBALS['filters']['counter'][$filterName][$filterFunction] = 0;

				// Mark this filter as loaded (from database)
				$GLOBALS['filters']['loaded'][$filterName][$filterFunction] = true;

				// Set this filter
				$GLOBALS['filters']['chains'][$filterName][$filterFunction] = $filterArray['filter_active'];

				// Is the array element for counter there?
				if (isset($filterArray['filter_counter'])) {
					// Then use this value!
					$GLOBALS['filters']['counter'][$filterName][$filterFunction] = $filterArray['filter_counter'];
				} // END - if
			} // END - while
		} // END - if

		// Free result
		SQL_FREERESULT($result);
	} // END - if

	// Init filters
	registerFilter('init', 'UPDATE_LOGIN_DATA');
	registerFilter('init', 'INIT_RANDOMIZER');

	// Login failures handler
	registerFilter('post_youhere_line', 'CALL_HANDLER_LOGIN_FAILTURES');

	// Filters for pre-extension-registration
	registerFilter('pre_extension_installed', 'RUN_SQLS');

	// Filters for post-extension-registration
	registerFilter('post_extension_installed', 'AUTO_ACTIVATE_EXTENSION');
	registerFilter('post_extension_installed', 'SOLVE_TASK');
	registerFilter('post_extension_installed', 'loadIncludeLUDES');
	registerFilter('post_extension_installed', 'REMOVE_UPDATES');

	// Solving tasks
	registerFilter('solve_task', 'SOLVE_TASK');

	// Loading includes in general
	registerFilter('load_includes', 'loadIncludeLUDES');

	// Run SQLs
	registerFilter('run_sqls', 'RUN_SQLS');

	// Admin ACL check
	registerFilter('check_admin_acl', 'CHECK_ADMIN_ACL');

	// Register shutdown filters
	registerFilter('shutdown', 'FLUSH_FILTERS');
}

// "Registers" a new filter function
function registerFilter ($filterName, $filterFunction, $silentAbort = true, $force = false, $dry_run = false) {
	// Extend the filter function name
	$filterFunction = sprintf("FILTER_%s", strtoupper($filterFunction));

	// Is that filter already there?
	if ((isset($GLOBALS['filters']['chains'][$filterName][$filterFunction])) && (!$force)) {
		// Then abort here
		if (!$silentAbort) {
			addFatalMessage(__FUNCTION__, __LINE__, getMessage('FILTER_FAILED_ALREADY_ADDED'), array($filterFunction, $filterName));
		} // END - if

		// Abort here
		return false;
	} // END - if

	// Is the function there?
	if (!function_exists($filterFunction)) {
		// Then abort here
		addFatalMessage(__FUNCTION__, __LINE__, getMessage('FILTER_FAILED_NOT_FOUND'), array($filterFunction, $filterName));
		return false;
	} // END - if

	// Shall we add it?
	if (!$dry_run) {
		// Simply add it to the array
		$GLOBALS['filters']['chains'][$filterName][$filterFunction] = 'Y';
		$GLOBALS['filters']['counter'][$filterName][$filterFunction] = 0;
	} // END - if
}

// "Unregisters" a filter from the given chain
function unregisterFilter ($filterName, $filterFunction, $force = false, $dry_run = false) {
	// Extend the filter function name only if not loaded from database
	if (!isset($GLOBALS['filters']['loaded'][$filterName][$filterFunction])) {
		$filterFunction = sprintf("FILTER_%s", strtoupper($filterFunction));
	} // END - if

	// Is that filter there?
	if ((!isset($GLOBALS['filters']['chains'][$filterName][$filterFunction])) && (!$force)) {
		// Not found, so abort here
		addFatalMessage(__FUNCTION__, __LINE__, getMessage('FILTER_FAILED_NOT_REMOVED'), array($filterFunction, $filterName));
		return false;
	} // END - if

	// Shall we remove? (default, not while just showing an extension removal)
	if (!$dry_run) {
		// Mark for filter removal
		$GLOBALS['filters']['chains'][$filterName][$filterFunction] = "R";
		unset($GLOBALS['filters']['counter'][$filterName][$filterFunction]);
	} // END  - if
}

// "Runs" the given filters, data is optional and can be any type of data
function runFilterChain ($filterName, $data = null, $silentAbort = true) {
	// Is that filter chain there?
	if (!isset($GLOBALS['filters']['chains'][$filterName])) {
		// Then abort here (quick'N'dirty hack)
		if ((!$silentAbort) && (defined('FILTER_FAILED_NO_FILTER_FOUND'))) {
			// Add fatal message
			addFatalMessage(__FUNCTION__, __LINE__, getMessage('FILTER_FAILED_NO_FILTER_FOUND'), $filterName);
		} // END - if

		// Abort here
		return false;
	} // END - if

	// Default return value
	$returnValue = $data;

	// Then run all filters
	foreach ($GLOBALS['filters']['chains'][$filterName] as $filterFunction=>$active) {
		// Debug message
		//* DEBUG: */ echo __FUNCTION__."(<font color=\"#0000aa\">".__LINE__."</font>): name={$filterName},func={$filterFunction},active={$active}<br />\n";

		// Is the filter active?
		if ($active == 'Y') {
			// Is this filter there?
			if (!function_exists($filterFunction)) {
				// Unregister it
				unregisterFilter($filterName, $filterFunction);

				// Skip this entry
				continue;
			} // END - if

			// Call the filter chain
			$returnValue = call_user_func_array($filterFunction, array($returnValue));

			// Update usage counter
			$GLOBALS['filters']['counter'][$filterName][$filterFunction]++;
		} // END - if
	} // END - foreach

	// Return the filtered content
	return $returnValue;
}

// -----------------------------------------------------------------------------
// Generic filter functions we always need
// -----------------------------------------------------------------------------

// Filter for flushing all new filters to the database
function FILTER_FLUSH_FILTERS () {
	// Clear all previous SQL queries
	INIT_SQLS();

	// Are we installing?
	if ((isInstalling()) || (!isInstalled())) {
		// Then silently skip this filter
		return true;
	} // END - if

	// Is a database link here and not in installation mode?
	if ((!SQL_IS_LINK_UP()) && (!isInstalling())) {
		// Abort here
		addFatalMessage(__FUNCTION__, __LINE__, getMessage('FILTER_FLUSH_FAILED_NO_DATABASE'));
		return false;
	} // END - if

	// Is the extension sql_patches updated?
	if (EXT_VERSION_IS_OLDER('sql_patches', '0.5.9')) {
		// Abort silently here
		return false;
	} // END - if

	// Nothing is added/remove by default
	$inserted = 0;
	$removed = 0;

	// Prepare SQL queries
	$insertSQL = "INSERT INTO `{!_MYSQL_PREFIX!}_filters` (`filter_name`,`filter_function`,`filter_active`) VALUES";
	$removeSQL = "DELETE LOW_PRIORITY FROM `{!_MYSQL_PREFIX!}_filters` WHERE";

	// Write all filters to database
	foreach ($GLOBALS['filters']['chains'] as $filterName => $filterArray) {
		// Walk through all filters
		foreach ($filterArray as $filterFunction => $active) {
			// Is this filter loaded?
			if (!isset($GLOBALS['filters']['loaded'][$filterName][$filterFunction])) {
				// Add this filter (all filters are active by default)
				$insertSQL .= sprintf("('%s','%s','Y'),", $filterName, $filterFunction);
				$inserted++;
			} elseif ($active == "R") {
				// Remove this filter
				$removeSQL .= sprintf(" (`filter_name`='%s' AND `filter_function`='%s') OR", $filterName, $filterFunction);
				$removed++;
			}
		} // END - foreach
	} // END - foreach

	// Something has been added?
	if ($inserted > 0) {
		// Finish SQL command
		$insertSQL = substr($insertSQL, 0, -1);

		// And run it
		ADD_SQL($insertSQL);
	} // END - if

	// Something has been removed?
	if ($removed > 0) {
		// Finish SQL command
		$removeSQL = substr($removeSQL, 0, -2) . "LIMIT ".$removed;

		// And run it
		ADD_SQL($removeSQL);
	} // END - if

	// Shall we update usage counters (ONLY FOR DEBUGGING!)
	if (getConfig('update_filter_usage') == 'Y') {
		// Update all counters
		foreach ($GLOBALS['filters']['counter'] as $filterName => $filterArray) {
			// Walk through all filters
			foreach ($filterArray as $filterFunction => $cnt) {
				// Construct and add the query
				ADD_SQL(sprintf("UPDATE `{!_MYSQL_PREFIX!}_filters` SET `filter_counter`=%s WHERE `filter_name`='%s' AND `filter_function`='%s' LIMIT 1",
				bigintval($cnt),
				$filterName,
				$filterFunction
				));
			} // END - foreach
		} // END - foreach
	} // END - if

	// Run the run_sqls filter in non-dry mode
	runFilterChain('run_sqls');
}

// Filter for calling the handler for login failures
function FILTER_CALL_HANDLER_LOGIN_FAILTURES ($data) {
	// Init content
	$content = $data;

	// Handle failed logins here if not in guest
	//* DEBUG: */ print __FUNCTION__."(<font color=\"#0000aa\">".__LINE__."</font>):type={$data['type']},action={$GLOBALS['action']},what={$GLOBALS['what']},lvl={$data['access_level']}<br />\n";
	if ((($data['type'] == 'what') || ($data['type'] == 'action') && ((!isset($GLOBALS['what'])) || ($GLOBALS['what'] == "overview") || ($GLOBALS['what'] == getConfig('index_home')))) && ($data['access_level'] != 'guest') && ((GET_EXT_VERSION('sql_patches') >= '0.4.7') || (GET_EXT_VERSION('admins') >= '0.7.0'))) {
		// Handle failure
		$content['content'] .= HANDLE_LOGIN_FAILTURES($data['access_level']);
	} // END - if

	// Return the content
	return $content;
}

// Filter for redirecting to logout if sql_patches has been installed
function FILTER_REDIRECT_TO_LOGOUT_SQL_PATCHES () {
	// Remove this filter
	unregisterFilter('shutdown', __FUNCTION__);

	// Is the element set?
	if (isset($GLOBALS['ext_load_mode'])) {
		// Redirect here
		redirectToUrl('modules.php?module=admin&amp;logout=1&amp;' . $GLOBALS['ext_load_mode'] . '=sql_patches');
	} // END - if

	// This should not happen!
	DEBUG_LOG(__FUNCTION__, __LINE__, 'Cannot auto-logout because no extension load-mode has been set.');
}

// Filter for auto-activation of a extension
function FILTER_AUTO_ACTIVATE_EXTENSION ($data) {
	// Is this extension always activated?
	if (EXT_GET_ALWAYS_ACTIVE() == 'Y') {
		// Then activate the extension
		//* DEBUG: */ echo __FUNCTION__."(<font color=\"#0000aa\">".__LINE__."</font>): ext_name={$data['ext_name']}<br />\n";
		ACTIVATE_EXTENSION($data['ext_name']);
	} // END - if

	// Return the data
	return $data;
}

// Filter for solving task given task
function FILTER_SOLVE_TASK ($data) {
	// Don't solve anything if no admin!
	if (!IS_ADMIN()) return $data;

	// Is this a direct task id or array element task_id is found?
	if (is_int($data)) {
		// Then solve it...
		ADMIN_SOLVE_TASK($data);
	} elseif ((is_array($data)) && (isset($data['task_id']))) {
		// Solve it...
		ADMIN_SOLVE_TASK($data['task_id']);
	}

	// Return the data
	return $data;
}

// Filter to load include files
function FILTER_loadIncludeLUDES () {
	// Default is $data as inclusion list
	$data = GET_INC_POOL();

	// Is it an array?
	if ((!isset($data)) || (!is_array($data))) {
		// Then abort here
		debug_report_bug(sprintf("INC_POOL is no array! Type: %s", gettype($data)));
	} elseif (isset($data['inc_pool'])) {
		// Use this as new inclusion pool!
		SET_INC_POOL($data['inc_pool']);
	}

	// Check for added include files
	if (COUNT_INC_POOL() > 0) {
		// Loads every include file
		foreach (GET_INC_POOL() as $FQFN) {
			loadIncludeOnce($FQFN);
		} // END - foreach

		// Reset array
		INIT_INC_POOL();
	} // END - if

	// Continue with processing
	return $data;
}

// Filter for running SQL commands
function FILTER_RUN_SQLS ($data) {
	// Debug message
	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, " - Entered!");

	// Is the array there?
	if ((IS_SQLS_VALID()) && ((!isset($data['dry_run'])) || ($data['dry_run'] == false))) {
		// Run SQL commands
		//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, " - Found ".COUNT_SQLS()." queries to run.");
		foreach (GET_SQLS() as $sql) {
			// Trim spaces away
			$sql = trim($sql);

			// Is there still a query left?
			if (!empty($sql)) {
				// Do we have an "ALTER TABLE" command?
				if (substr(strtolower($sql), 0, 11) == "alter table") {
					// Analyse the alteration command
					//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "Alterting table: {$sql}");
					SQL_ALTER_TABLE($sql, __FUNCTION__, __LINE__);
				} else {
					// Run regular SQL command
					//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, "Running regular query: {$sql}");
					SQL_QUERY($sql, __FUNCTION__, __LINE__, false);
				}
			} // END - if
		} // END - foreach
	} // END - if

	// Debug message
	//* DEBUG: */ DEBUG_LOG(__FUNCTION__, __LINE__, " - Left!");
}

// Filter for updating/validating login data
function FILTER_UPDATE_LOGIN_DATA () {
	// Add missing array
	if ((!isset($GLOBALS['last'])) || (!is_array($GLOBALS['last']))) $GLOBALS['last'] = array();

	// Recheck if logged in
	if (!IS_MEMBER()) return false;

	// Secure user ID
	setUserId(getSession('userid'));

	// Load last module and last online time
	$result = SQL_QUERY_ESC("SELECT last_module, last_online FROM `{!_MYSQL_PREFIX!}_user_data` WHERE userid=%s LIMIT 1",
		array(getUserId()), __FUNCTION__, __LINE__);

	// Entry found?
	if (SQL_NUMROWS($result) == 1) {
		// Load last module and online time
		list($mod, $onl) = SQL_FETCHROW($result);

		// Maybe first login time?
		if (empty($mod)) $mod = "login";

		// This will be displayed on welcome page! :-)
		if (empty($GLOBALS['last']['module'])) {
			$GLOBALS['last']['module'] = $mod; $GLOBALS['last']['online'] = $onl;
		} // END - if

		// 'what' not set?
		if (empty($GLOBALS['what'])) {
			// Fix it to default
			$GLOBALS['what'] = "welcome";
			if (getConfig('index_home') != '') $GLOBALS['what'] = getConfig('index_home');
		} // END - if

		// Update last module / online time
		SQL_QUERY_ESC("UPDATE `{!_MYSQL_PREFIX!}_user_data` SET `last_module`='%s', last_online=UNIX_TIMESTAMP(), REMOTE_ADDR='%s' WHERE userid=%s LIMIT 1",
			array($GLOBALS['what'], detectRemoteAddr(), getUserId()), __FUNCTION__, __LINE__);
	}  else {
		// Destroy session, we cannot update!
		destroyUserSession();
	}

	// Free the result
	SQL_FREERESULT($result);
}

// Filter for checking admin ACL
function FILTER_CHECK_ADMIN_ACL () {
	// Extension not installed so it's always allowed to access everywhere!
	$ret = true;

	// Ok, Cookie-Update done
	if ((GET_EXT_VERSION('admins') >= '0.3.0') && (EXT_IS_ACTIVE('admins'))) {
		// Check if action GET variable was set
		$action = SQL_ESCAPE($GLOBALS['action']);
		if (!empty($GLOBALS['what'])) {
			// Get action value by what-value
			$action = getModeAction('admin', $GLOBALS['what']);
		} // END - if

		// Check for access control line of current menu entry
		$ret = adminsCheckAdminAcl($action, $GLOBALS['what']);
	} // END - if

	// Return result
	return $ret;
}

// Filter for initializing randomizer
function FILTER_INIT_RANDOMIZER () {
	// Simply init the randomizer with seed and _ADD value
	mt_srand(generateSeed() + getConfig('_ADD'));
}

// Filter for removing updates
function FILTER_REMOVE_UPDATES () {
	// Init removal list
	EXT_INIT_REMOVAL_LIST();

	// Add the current extension to it
	EXT_ADD_CURRENT_TO_REMOVAL_LIST();

	// Simply remove it
	UNSET_EXT_SQLS();

	// Do we need to remove update depency?
	if (EXT_COUNT_UPDATE_DEPENDS() > 0) {
		// Then find all updates we shall no longer execute
		foreach (EXT_GET_UPDATE_DEPENDS() as $id=>$ext_name) {
			// Shall we remove this update?
			if (in_array($ext_name, EXT_GET_REMOVAL_LIST())) {
				// Then remove this extension!
				EXT_REMOVE_UPDATE_DEPENDS($ext_name);
			} // END - if
		} // END - foreach
	} // END - if
}

//
?>
