<?php
/************************************************************************
 * MXChange v0.2.1                                    Start: 08/25/2003 *
 * ===============                              Last change: 07/01/2005 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : modules.php                                      *
 * -------------------------------------------------------------------- *
 * Short description : Main loader file. Loads our needed stuff         *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Hauptladedatei. Laedt alle benoetigten Dateien   *
 * -------------------------------------------------------------------- *
 * $Revision::                                                        $ *
 * $Date::                                                            $ *
 * $Tag:: 0.2.1-FINAL                                                 $ *
 * $Author::                                                          $ *
 * Needs to be in all Files and every File needs "svn propset           *
 * svn:keywords Date Revision" (autoprobset!) at least!!!!!!            *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2008 by Roland Haeder                           *
 * For more information visit: http://www.mxchange.org                  *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// XDEBUG call
//xdebug_start_trace();

// Load security stuff here
require('inc/libs/security_functions.php');

// Init start time
$GLOBALS['startTime'] = microtime(true);

// Init output mode and module
$GLOBALS['output_mode'] = 0;
$GLOBALS['module'] = '';

// Needed include files
require('inc/config-global.php');

// Fix missing module to 'index'
if (!REQUEST_ISSET_GET('module')) REQUEST_SET_GET('module', 'index');

// Check if logged in
if (IS_MEMBER()) {
	// Is still logged in so we welcome him with his name
	$result = SQL_QUERY_ESC("SELECT surname, family FROM `{!_MYSQL_PREFIX!}_user_data` WHERE userid=%s LIMIT 1",
	array(getUserId()), __FILE__, __LINE__);
	if (SQL_NUMROWS($result) == 1) {
		// Load surname and family's name and build the username
		list($s, $f) = SQL_FETCHROW($result);
		$username = $s . ' ' . $f;

		// Additionally admin?
		if (IS_ADMIN()) {
			// Add it
			$username .= ' ({--_ADMIN_SHORT--})';
		} // END - if
	} else {
		// Hmmm, logged in and no valid userid?
		$username = '<em>{--_UNKNOWN--}</em>';

		// Destroy session
		destroyUserSession();

		// Kill userid
		setUserId(0);
	}

	// Free memory
	SQL_FREERESULT($result);
} elseif (IS_ADMIN()) {
	// Admin is there
	$username = getMessage('_ADMIN');
} else {
	// He's a guest, hello there... ;-)
	$username = getMessage('_GUEST');
}

// The header file
loadIncludeOnce('inc/header.php');

// Modules are by default not valid!
$MOD_VALID = false; $check = 'failed';
if ((getConfig('maintenance') == 'Y') && (!IS_ADMIN()) && ($GLOBALS['module'] != 'admin')) {
	// Maintain mode is active and you are no admin
	addFatalMessage(__FILE__, __LINE__, getMessage('LANG_DOWN_MAINTAINCE'));
} elseif ((SQL_IS_LINK_UP()) && (getTotalFatalErrors() == 0)) {
	// Construct module name
	define('__MODULE', sprintf("inc/modules/%s.php", SQL_ESCAPE($GLOBALS['module'])));

	// Did we found the module listed in allowed modules and are we successfully connected?
	$check = checkModulePermissions($GLOBALS['module']);
	switch ($check)
	{
		case 'admin_only':
		case 'mem_only':
		case 'done':
			// Does the module exists on local file system?
			if ((isFileReadable(constant('__MODULE'))) && (getTotalFatalErrors() == 0)) {
				// Module is valid, active and located on the local disc...
				$MOD_VALID = true;
			} elseif (!empty($URL)) {
				// An URL was specified so we load the de-referrer module
				redirectToUrl(DEREFERER($URL));
			} elseif (getTotalFatalErrors() == 0) {
				addFatalMessage(__FILE__, __LINE__, sprintf(getMessage('LANG_MOD_REG_404'), $GLOBALS['module']));
			}
			break;

		case '404':
			addFatalMessage(__FILE__, __LINE__, sprintf(getMessage('LANG_MOD_REG_404'), $GLOBALS['module']));
			break;

		case 'locked':
			if (!isFileReadable(constant('__MODULE'))) {
				// Module does addionally not exists
				addFatalMessage(__FILE__, __LINE__, sprintf(getMessage('LANG_MOD_REG_404'), $GLOBALS['module']));
			} // END - if

			// Add fatal message
			addFatalMessage(__FILE__, __LINE__, sprintf(getMessage('LANG_MOD_REG_LOCKED'), $GLOBALS['module']));
			break;

		default:
			DEBUG_LOG(__FILE__, __LINE__, sprintf("Unknown status %s return from module check. Module=%s", $check, $GLOBALS['module']));
			addFatalMessage(__FILE__, __LINE__, sprintf(getMessage('LANG_MOD_REG_UNKNOWN'), $check));
			break;
	}
} elseif (getTotalFatalErrors() == 0) {
	// MySQL problems!
	addFatalMessage(__FILE__, __LINE__, getMessage('MYSQL_ERRORS'));
}

if (($MOD_VALID) && (defined('__MODULE'))) {
	/////////////////////////////////////////////
	// Main including line DO NOT REMOVE/EDIT! //
	/////////////////////////////////////////////
	//
	// Everything is okay so we can load the module
	loadIncludeOnce(constant('__MODULE'));
} // END - if

// Next-to-end add the footer
loadIncludeOnce('inc/footer.php');

//
?>
